/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-

   nautilus-window-slot.c: Nautilus window slot
 
   Copyright (C) 2008 Free Software Foundation, Inc.
  
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.
  
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
  
   You should have received a copy of the GNU General Public
   License along with this program; if not, write to the
   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
  
   Author: Christian Neumair <cneumair@gnome.org>
*/
#include <config.h>
#include "nautilus-window-slot.h"
#include "nautilus-navigation-window-slot.h"

#include "nautilus-desktop-window.h"
#include "nautilus-window-private.h"
#include "nautilus-window-manage-views.h"
#include <libnautilus-private/nautilus-file.h>
#include <libnautilus-private/nautilus-file-utilities.h>
#include <libnautilus-private/nautilus-window-slot-info.h>
#include <libnautilus-private/nautilus-global-preferences.h>
#include <eel/eel-gtk-macros.h>
#include <eel/eel-string.h>
#include <vte/vte.h>

static void nautilus_window_slot_init       (NautilusWindowSlot *slot);
static void nautilus_window_slot_class_init (NautilusWindowSlotClass *class);
static void nautilus_window_slot_dispose    (GObject *object);

static void nautilus_window_slot_info_iface_init (NautilusWindowSlotInfoIface *iface);

G_DEFINE_TYPE_WITH_CODE (NautilusWindowSlot,
			 nautilus_window_slot,
			 G_TYPE_OBJECT,
			 G_IMPLEMENT_INTERFACE (NAUTILUS_TYPE_WINDOW_SLOT_INFO,
						nautilus_window_slot_info_iface_init))
#define parent_class nautilus_window_slot_parent_class

#define TERMINAL_PALETTE_SIZE 16

static void
query_editor_changed_callback (NautilusSearchBar *bar,
			       NautilusQuery *query,
			       gboolean reload,
			       NautilusWindowSlot *slot)
{
	NautilusDirectory *directory;

	directory = nautilus_directory_get_for_file (slot->viewed_file);
	g_assert (NAUTILUS_IS_SEARCH_DIRECTORY (directory));

	nautilus_search_directory_set_query (NAUTILUS_SEARCH_DIRECTORY (directory),
					     query);
	if (reload) {
		nautilus_window_slot_reload (slot);
	}

	nautilus_directory_unref (directory);
}

static void
real_update_query_editor (NautilusWindowSlot *slot)
{
	GtkWidget *query_editor;
	NautilusQuery *query;
	NautilusDirectory *directory;
	NautilusSearchDirectory *search_directory;

	directory = nautilus_directory_get (slot->location);

	if (NAUTILUS_IS_SEARCH_DIRECTORY (directory)) {
		search_directory = NAUTILUS_SEARCH_DIRECTORY (directory);

		query_editor = nautilus_query_editor_new (nautilus_search_directory_is_saved_search (search_directory),
							  nautilus_search_directory_is_indexed (search_directory));

		slot->query_editor = NAUTILUS_QUERY_EDITOR (query_editor);

		nautilus_window_slot_add_extra_location_widget (slot, query_editor);
		gtk_widget_show (query_editor);
		g_signal_connect_object (query_editor, "changed",
					 G_CALLBACK (query_editor_changed_callback), slot, 0);
		
		query = nautilus_search_directory_get_query (search_directory);
		if (query != NULL) {
			nautilus_query_editor_set_query (NAUTILUS_QUERY_EDITOR (query_editor),
							 query);
			g_object_unref (query);
		} else {
			nautilus_query_editor_set_default_query (NAUTILUS_QUERY_EDITOR (query_editor));
		}
	} 

	nautilus_directory_unref (directory);
}


static void
real_active (NautilusWindowSlot *slot)
{
	NautilusWindow *window;

	window = slot->pane->window;

	/* sync window to new slot */
	nautilus_window_sync_status (window);
	nautilus_window_sync_allow_stop (window, slot);
	nautilus_window_sync_title (window, slot);
	nautilus_window_sync_zoom_widgets (window);
	nautilus_window_pane_sync_location_widgets (slot->pane);
	nautilus_window_pane_sync_search_widgets (slot->pane);

	if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_ZOOM_SLIDER)) {
		zoom_slider_update_value (window);
	}

	if (slot->viewed_file != NULL) {
		nautilus_window_load_view_as_menus (window);
		nautilus_window_load_extension_menus (window);
	}
}

static void
nautilus_window_slot_active (NautilusWindowSlot *slot)
{
	NautilusWindow *window;
	NautilusWindowPane *pane;

	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));

	pane = NAUTILUS_WINDOW_PANE (slot->pane);
	window = NAUTILUS_WINDOW (slot->pane->window);
	g_assert (g_list_find (pane->slots, slot) != NULL);
	g_assert (slot == window->details->active_pane->active_slot);

	EEL_CALL_METHOD (NAUTILUS_WINDOW_SLOT_CLASS, slot,
			 active, (slot));
}

static void
real_inactive (NautilusWindowSlot *slot)
{
	NautilusWindow *window;

	window = NAUTILUS_WINDOW (slot->pane->window);
	g_assert (slot == window->details->active_pane->active_slot);
}

static void
nautilus_window_slot_inactive (NautilusWindowSlot *slot)
{
	NautilusWindow *window;
	NautilusWindowPane *pane;

	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));

	pane = NAUTILUS_WINDOW_PANE (slot->pane);
	window = NAUTILUS_WINDOW (pane->window);

	g_assert (g_list_find (pane->slots, slot) != NULL);
	g_assert (slot == window->details->active_pane->active_slot);

	EEL_CALL_METHOD (NAUTILUS_WINDOW_SLOT_CLASS, slot,
			 inactive, (slot));
}


static void
nautilus_window_slot_init (NautilusWindowSlot *slot)
{
	GtkWidget *content_box, *eventbox, *extras_vbox;

	content_box = gtk_vbox_new (FALSE, 0);
	slot->content_box = content_box;
	gtk_widget_show (content_box);

	eventbox = gtk_event_box_new ();
        slot->extra_location_event_box = eventbox;
	gtk_widget_set_name (eventbox, "nautilus-extra-view-widget");
        gtk_box_pack_start (GTK_BOX (content_box), eventbox, FALSE, FALSE, 0);

	extras_vbox = gtk_vbox_new (FALSE, 6);
	gtk_container_set_border_width (GTK_CONTAINER (extras_vbox), 6);
	slot->extra_location_widgets = extras_vbox;
	gtk_container_add (GTK_CONTAINER (eventbox), extras_vbox);
	gtk_widget_show (extras_vbox);

	slot->view_box = gtk_vbox_new (FALSE, 0);
	gtk_box_pack_start (GTK_BOX (content_box), slot->view_box, TRUE, TRUE, 0);
	gtk_widget_show (slot->view_box);

	slot->title = g_strdup (_("Loading..."));
}

static void
nautilus_window_slot_class_init (NautilusWindowSlotClass *class)
{
	class->active = real_active;
	class->inactive = real_inactive;
	class->update_query_editor = real_update_query_editor; 

	G_OBJECT_CLASS (class)->dispose = nautilus_window_slot_dispose;
}

static int
nautilus_window_slot_get_selection_count (NautilusWindowSlot *slot)
{
	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));

	if (slot->content_view != NULL) {
		return nautilus_view_get_selection_count (slot->content_view);
	}
	return 0;
}

GFile *
nautilus_window_slot_get_location (NautilusWindowSlot *slot)
{
	g_assert (slot != NULL);
	g_assert (NAUTILUS_IS_WINDOW (slot->pane->window));

	if (slot->location != NULL) {
		return g_object_ref (slot->location);
	}
	return NULL;
}

char *
nautilus_window_slot_get_location_uri (NautilusWindowSlotInfo *slot)
{
	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));

	if (slot->location) {
		return g_file_get_uri (slot->location);
	}
	return NULL;
}

static void
nautilus_window_slot_make_hosting_pane_active (NautilusWindowSlot *slot)
{
	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
	g_assert (NAUTILUS_IS_WINDOW_PANE (slot->pane));
	
	nautilus_window_set_active_slot (slot->pane->window, slot);
}

char *
nautilus_window_slot_get_title (NautilusWindowSlot *slot)
{
	char *title;

	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));

	title = NULL;
	if (slot->new_content_view != NULL) {
		title = nautilus_view_get_title (slot->new_content_view);
	} else if (slot->content_view != NULL) {
		title = nautilus_view_get_title (slot->content_view);
	}

	if (title == NULL) {
		title = nautilus_compute_title_for_location (slot->location);
	}

	return title;
}

static NautilusWindow *
nautilus_window_slot_get_window (NautilusWindowSlot *slot)
{
	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));
	return slot->pane->window;
}

/* nautilus_window_slot_set_title:
 *
 * Sets slot->title, and if it changed
 * synchronizes the actual GtkWindow title which
 * might look a bit different (e.g. with "file browser:" added)
 */
static void
nautilus_window_slot_set_title (NautilusWindowSlot *slot,
				const char *title)
{
	NautilusWindow *window;
	gboolean changed;

	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));

	window = NAUTILUS_WINDOW (slot->pane->window);

	changed = FALSE;

	if (eel_strcmp (title, slot->title) != 0) {
		changed = TRUE;

		g_free (slot->title);
		slot->title = g_strdup (title);
	}

	if (eel_strlen (slot->title) > 0 && slot->current_location_bookmark &&
	    nautilus_bookmark_set_name (slot->current_location_bookmark,
					slot->title)) {
		changed = TRUE;

		/* Name of item in history list changed, tell listeners. */
		nautilus_send_history_list_changed ();
	}

	if (changed) {
		nautilus_window_sync_title (window, slot);
	}
}


/* nautilus_window_slot_update_title:
 * 
 * Re-calculate the slot title.
 * Called when the location or view has changed.
 * @slot: The NautilusWindowSlot in question.
 * 
 */
void
nautilus_window_slot_update_title (NautilusWindowSlot *slot)
{
	char *title;

	title = nautilus_window_slot_get_title (slot);
	nautilus_window_slot_set_title (slot, title);
	g_free (title);
}

/* nautilus_window_slot_update_icon:
 * 
 * Re-calculate the slot icon
 * Called when the location or view or icon set has changed.
 * @slot: The NautilusWindowSlot in question.
 */
void
nautilus_window_slot_update_icon (NautilusWindowSlot *slot)
{
	NautilusWindow *window;
	NautilusIconInfo *info;
	const char *icon_name;
	GdkPixbuf *pixbuf;

	window = slot->pane->window;

	g_return_if_fail (NAUTILUS_IS_WINDOW (window));

	info = EEL_CALL_METHOD_WITH_RETURN_VALUE (NAUTILUS_WINDOW_CLASS, window,
						 get_icon, (window, slot));

	icon_name = NULL;
	if (info) {
		icon_name = nautilus_icon_info_get_used_name (info);
		if (icon_name != NULL) {
			/* Gtk+ doesn't short circuit this (yet), so avoid lots of work
			 * if we're setting to the same icon. This happens a lot e.g. when
			 * the trash directory changes due to the file count changing.
			 */
			if (g_strcmp0 (icon_name, gtk_window_get_icon_name (GTK_WINDOW (window))) != 0) {			
				gtk_window_set_icon_name (GTK_WINDOW (window), icon_name);
			}
		} else {
			pixbuf = nautilus_icon_info_get_pixbuf_nodefault (info);
			
			if (pixbuf) {
				gtk_window_set_icon (GTK_WINDOW (window), pixbuf);
				g_object_unref (pixbuf);
			} 
		}
		
		g_object_unref (info);
	}
        //nautilus_navigation_window_pane_update_zoom_slider (slot->pane);
}

void
nautilus_window_slot_is_in_active_pane (NautilusWindowSlot *slot,
					gboolean is_active)
{
	/* NULL is valid, and happens during init */
	if (!slot) {
		return;
	}

	/* it may also be that the content is not a valid directory view during init */
	if (slot->content_view != NULL) {
		nautilus_view_set_is_active (slot->content_view, is_active);
	}

	if (slot->new_content_view != NULL) {
		nautilus_view_set_is_active (slot->new_content_view, is_active);
	}
}

void
nautilus_window_slot_connect_content_view (NautilusWindowSlot *slot,
					   NautilusView *view)
{
	NautilusWindow *window;

	window = slot->pane->window;
	if (window != NULL && slot == nautilus_window_get_active_slot (window)) {
		nautilus_window_connect_content_view (window, view);
	}
}

void
nautilus_window_slot_disconnect_content_view (NautilusWindowSlot *slot,
					      NautilusView *view)
{
	NautilusWindow *window;

	window = slot->pane->window;
	if (window != NULL && window->details->active_pane && window->details->active_pane->active_slot == slot) {
		nautilus_window_disconnect_content_view (window, view);
	}
}

void
nautilus_window_slot_update_eterm_path(NautilusWindowSlot *slot)
{
        if (slot->vte != NULL) {
                char *path = g_file_get_path (slot->location);
                gchar *cmd = g_strconcat("cd ", g_shell_quote(path), "\n", NULL);
                vte_terminal_feed_child(VTE_TERMINAL(slot->vte), cmd, strlen(cmd));
                g_free (path);
                g_free (cmd);
        }
}

static void
eterm_menu_copy_cb (GtkMenuItem *menuitem, gpointer user_data)
{
        VteTerminal *vte = VTE_TERMINAL(user_data);
        
        vte_terminal_copy_clipboard (vte);
}

static void
eterm_menu_paste_cb (GtkMenuItem *menuitem, gpointer user_data)
{
        VteTerminal *vte = VTE_TERMINAL(user_data);
       
        vte_terminal_paste_clipboard (vte);
}

static void
eterm_configure_cb (GtkMenuItem *menuitem, gpointer user_data)
{
        GdkScreen *screen = gdk_screen_get_default();
        char *cmd = "ne-terminal-config";
        gdk_spawn_command_line_on_screen (screen, cmd, NULL);
}

static void
eterm_popup_menu_show (GtkWidget *vte, GdkEventButton *event)
{
        GtkWidget *popup;
	GtkWidget *item;

        int button, event_time;
        gboolean has_selection, has_clipboard;

        popup = gtk_menu_new();

        has_selection = vte_terminal_get_has_selection (VTE_TERMINAL(vte));
        GtkClipboard *cb = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
        has_clipboard = gtk_clipboard_wait_is_text_available (cb);

	item = gtk_menu_item_new_with_mnemonic (_("Copy"));
	g_signal_connect (item, "activate",
			  G_CALLBACK (eterm_menu_copy_cb),
			  vte);
	gtk_menu_shell_append (GTK_MENU_SHELL (popup),
			       item);
	gtk_widget_set_sensitive (item, has_selection);

	item = gtk_menu_item_new_with_mnemonic (_("Paste"));
	g_signal_connect (item, "activate",
			  G_CALLBACK (eterm_menu_paste_cb),
			  vte);
	gtk_menu_shell_append (GTK_MENU_SHELL (popup),
			       item);
	gtk_widget_set_sensitive (item, has_clipboard);

	gtk_menu_shell_append (GTK_MENU_SHELL (popup),
			       gtk_separator_menu_item_new ());

	item = gtk_image_menu_item_new_with_mnemonic (_("Configure Terminal"));
	g_signal_connect (item, "activate",
			  G_CALLBACK (eterm_configure_cb), NULL);
	gtk_menu_shell_append (GTK_MENU_SHELL (popup),
			       item);
	gtk_widget_show_all (popup);

	if (event) {
		button = event->button;
		event_time = event->time;
	} else {
		button = 0;
		event_time = gtk_get_current_event_time ();
	}

	gtk_menu_attach_to_widget (GTK_MENU (popup),
				   vte,
				   NULL);

	gtk_menu_popup (GTK_MENU (popup), NULL, NULL, NULL, NULL,
			button, event_time);

}

static gboolean
eterm_button_press_cb (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	/*NautilusWindowSlotNavigationWindowPane *pane;
	NautilusNotebook *notebook;

	pane = NAUTILUS_NAVIGATION_WINDOW_PANE (user_data);*/
	if (GDK_BUTTON_PRESS == event->type && 3 == event->button) {
		eterm_popup_menu_show (widget, event);
		return TRUE;
	}
        
        return FALSE;
}

static void 
nautilus_window_slot_create_eterm(NautilusWindowSlot *slot)
{
        char *env[] = {"PATH=/usr/bin:/bin:./usr/local/bin:/bin", NULL}; 
        GPid child_pid = 0; 
        GError **error = NULL;
        int i;

        GtkWidget *vte1 = vte_terminal_new ();
        char *path = g_file_get_path (slot->location);
        
        char *shell = eel_preferences_get(NAUTILUS_PREFERENCES_ETERM_SHELL);
        char *params[] = {shell,NULL};

        /* Eterm preferences : cursor_shape */
        int cursor_shape = eel_preferences_get_integer (NAUTILUS_PREFERENCES_ETERM_CURSOR_SHAPE);
        switch (cursor_shape) {
        case 1:
                vte_terminal_set_cursor_shape (VTE_TERMINAL(vte1), VTE_CURSOR_SHAPE_IBEAM);
                break;
        case 2:
                vte_terminal_set_cursor_shape (VTE_TERMINAL(vte1), VTE_CURSOR_SHAPE_UNDERLINE);
                break;
        default:
                vte_terminal_set_cursor_shape (VTE_TERMINAL(vte1), VTE_CURSOR_SHAPE_BLOCK);
        }

        /* Eterm preferences: set colors */
        GdkColor text_color;
        char *gconf_text_color = eel_preferences_get(NAUTILUS_PREFERENCES_ETERM_TEXT_COLOR);
        if (!gdk_color_parse (gconf_text_color, &text_color))
                gdk_color_parse ("white", &text_color);
        g_free (gconf_text_color);

        GdkColor bg_color;
        char *gconf_bg_color = eel_preferences_get(NAUTILUS_PREFERENCES_ETERM_BG_COLOR);
        if (!gdk_color_parse (gconf_bg_color, &bg_color))
                gdk_color_parse ("black", &bg_color);
        g_free (gconf_bg_color);

        GdkColor colors[TERMINAL_PALETTE_SIZE];
        char *gconf_palette = eel_preferences_get (NAUTILUS_PREFERENCES_ETERM_COLORMAP);
        gchar **palette = g_strsplit(gconf_palette, ";", 0);
        for(i=0; i<TERMINAL_PALETTE_SIZE; i++) {
                gdk_color_parse(palette[i], &colors[i]);
        }
        g_strfreev(palette);
        g_free (gconf_palette);

        //vte_terminal_set_colors (VTE_TERMINAL(vte1), &text_color, &bg_color, NULL, 0);
        vte_terminal_set_colors (VTE_TERMINAL(vte1), &text_color, &bg_color, colors, TERMINAL_PALETTE_SIZE);

        /* Eterm preferences: set font */
        char *font = eel_preferences_get (NAUTILUS_PREFERENCES_ETERM_FONT);
        vte_terminal_set_font_from_string (VTE_TERMINAL(vte1), font);

        /* Eterm preferences: draw bold text
        ** terminal is bold by default */
        if (!eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ETERM_ALLOW_BOLD_TEXT))
                vte_terminal_set_allow_bold (VTE_TERMINAL(vte1), FALSE);

        /*g_signal_connect(GTK_OBJECT(vte1), "child-exited", 
                         G_CALLBACK(fork_done), NULL); 
        g_signal_connect(G_OBJECT(window), "delete_event", 
                         G_CALLBACK(window_deleted), vte1);*/ 

        vte_terminal_fork_command_full (VTE_TERMINAL(vte1),
                                        VTE_PTY_DEFAULT,
                                        path,
                                        params, env,
                                        G_SPAWN_FILE_AND_ARGV_ZERO,
                                        NULL, NULL,
                                        &child_pid, error);
        slot->vte = vte1;

        //amtest
        //printf("Child: %d\n", (int) child_pid);
        g_signal_connect (vte1,	"button_press_event",
                          G_CALLBACK (eterm_button_press_cb), NULL);

        g_free (path);
        g_free (shell);
}

void
nautilus_window_slot_show_eterm(NautilusWindowSlot *slot)
{
        int eterm_height;
        GtkWidget *scrolled;

        if (slot->mpane && !GTK_IS_WIDGET(slot->up))
        {
                //slot->up = nautilus_view_get_eterm_widget(slot->content_view);
                //slot->up = gtk_widget_new (GTK_TYPE_LABEL, "label", "Hello World", "xalign", 0.0, NULL);
                nautilus_window_slot_create_eterm(slot);
                if (eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ETERM_SCROLLBAR_SHOW)) { 
                        scrolled = gtk_scrolled_window_new (NULL, NULL);
        		gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
						        GTK_POLICY_AUTOMATIC,
						        GTK_POLICY_AUTOMATIC);
                        gtk_widget_show (scrolled);
                        gtk_container_add (GTK_CONTAINER (scrolled), slot->vte);
                        slot->up = scrolled;
                } else {   
                        slot->up = slot->vte;
                }
                eterm_height = eel_preferences_get_integer (NAUTILUS_PREFERENCES_ETERM_DEFAULT_HEIGHT);
                gtk_widget_set_size_request(GTK_WIDGET(slot->up), -1, eterm_height);
                gtk_paned_pack1(GTK_PANED(slot->mpane),
                                GTK_WIDGET(slot->up),
                                FALSE,
                                TRUE);
                gtk_widget_show_all (slot->up);
                slot->eterm_show = TRUE;
        }
}

void
nautilus_window_slot_hide_eterm(NautilusWindowSlot *slot)
{
        //nautilus_view_destroy_clutter_widget(slot->content_view);
        if (GTK_IS_WIDGET(slot->up)) {
                //nautilus_view_destroy_eterm_widget(slot->content_view);
                gtk_widget_destroy(slot->up);
                slot->up = NULL;
                slot->vte = NULL;
                /*if (nautilus_window_slot_get_clutter_view(slot)) {
                        nautilus_window_slot_show_clutter(slot);

                }*/
                slot->eterm_show = FALSE;
        }
}

//amtest
void
nautilus_window_slot_show_clutter(NautilusWindowSlot *slot)
{
        if (slot->mpane && !GTK_IS_WIDGET(slot->up))
        {
                slot->up = nautilus_view_get_clutter_widget(slot->content_view);

                gtk_widget_set_size_request(GTK_WIDGET(slot->up), -1, 200);
                gtk_paned_pack1(GTK_PANED(slot->mpane),
                                GTK_WIDGET(slot->up),
                                FALSE,
                                FALSE);
                gtk_widget_show_all (slot->up);
                slot->clutter_show = TRUE;
        }
}

void
nautilus_window_slot_hide_clutter(NautilusWindowSlot *slot)
{
        //nautilus_view_destroy_clutter_widget(slot->content_view);
        if (GTK_IS_WIDGET(slot->up) && slot->clutter_show) {
                nautilus_view_destroy_clutter_widget(slot->content_view);
                gtk_widget_destroy(slot->up);
                slot->up = NULL;
        //if (!slot->eterm_show)
                slot->clutter_show = FALSE;
        }
}

void
nautilus_window_slot_set_content_view_widget (NautilusWindowSlot *slot,
					      NautilusView *new_view)
{
	NautilusWindow *window;
	GtkWidget *widget;

	window = slot->pane->window;
	g_assert (NAUTILUS_IS_WINDOW (window));

        #if ENABLE_CLUTTER_VIEW
        /* duplicate coverflow stuff from old_view to new_view */
        if (slot->content_view != NULL && new_view != NULL) {
                if (slot->clutter_show) {
                        /* reuse coverflow */
                        nautilus_view_reuse_coverflow(new_view, slot->content_view);
                }
        }
        #endif

	if (slot->content_view != NULL) {
		/* disconnect old view */
		nautilus_window_slot_disconnect_content_view (slot, slot->content_view);

		widget = nautilus_view_get_widget (slot->content_view);
		gtk_widget_destroy (widget);
		g_object_unref (slot->content_view);
		slot->content_view = NULL;
	}

	if (new_view != NULL) {
                //#if ENABLE_CLUTTER_VIEW
                //amtest
                /*char *view_id =  nautilus_file_get_metadata 
				(nautilus_file_get(nautilus_window_slot_get_location(slot)), NAUTILUS_METADATA_KEY_DEFAULT_VIEW, NULL);*/
                //NautilusFile *file = nautilus_file_get (slot->location);

		slot->content_view = new_view;
                if(!slot->mpane)
                {
                    slot->mpane = gtk_vpaned_new ();
		    gtk_container_add (GTK_CONTAINER (slot->view_box),
                                       GTK_WIDGET (slot->mpane));
                }
                #if ENABLE_CLUTTER_VIEW
                if (slot->clutter_show && eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_CLUTTER))
                        nautilus_window_slot_show_clutter(slot);
                else
                        nautilus_window_slot_hide_clutter(slot);
                #endif

               	gtk_paned_pack2(GTK_PANED(slot->mpane),
                                GTK_WIDGET(new_view),
                                TRUE,
                                TRUE);
                
                gtk_widget_show_all (slot->mpane);
                //#endif

                /*#ifndef ENABLE_CLUTTER_VIEW
		gtk_container_add (GTK_CONTAINER (slot->view_box),
						  GTK_WIDGET (new_view));
                #endif*/

		slot->content_view = new_view;
		g_object_ref (slot->content_view);

		/* connect new view */
		nautilus_window_slot_connect_content_view (slot, new_view);
	}
}

void
nautilus_window_slot_set_allow_stop (NautilusWindowSlot *slot,
				     gboolean allow)
{
	NautilusWindow *window;

	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));

	slot->allow_stop = allow;

	window = NAUTILUS_WINDOW (slot->pane->window);
	nautilus_window_sync_allow_stop (window, slot);
}

void
nautilus_window_slot_set_status (NautilusWindowSlot *slot,
				 const char *status)
{
	NautilusWindow *window;

	g_assert (NAUTILUS_IS_WINDOW_SLOT (slot));

	g_free (slot->status_text);
	slot->status_text = g_strdup (status);

	window = NAUTILUS_WINDOW (slot->pane->window);
	if (slot == window->details->active_pane->active_slot) {
		nautilus_window_sync_status (window);
	}
}

/* nautilus_window_slot_update_query_editor:
 * 
 * Update the query editor.
 * Called when the location has changed.
 *
 * @slot: The NautilusWindowSlot in question.
 */
void
nautilus_window_slot_update_query_editor (NautilusWindowSlot *slot)
{
	if (slot->query_editor != NULL) {
		gtk_widget_destroy (GTK_WIDGET (slot->query_editor));
		g_assert (slot->query_editor == NULL);
	}

        //printf("^^^^%s\n", G_STRFUNC);
        EEL_CALL_METHOD (NAUTILUS_WINDOW_SLOT_CLASS, slot,
                         update_query_editor, (slot));

	eel_add_weak_pointer (&slot->query_editor);
}

static void
remove_all (GtkWidget *widget,
	    gpointer data)
{
	GtkContainer *container;
	container = GTK_CONTAINER (data);

	gtk_container_remove (container, widget);
}

void
nautilus_window_slot_remove_extra_location_widgets (NautilusWindowSlot *slot)
{
	gtk_container_foreach (GTK_CONTAINER (slot->extra_location_widgets),
			       remove_all,
			       slot->extra_location_widgets);
	gtk_widget_hide (slot->extra_location_event_box);
}

void
nautilus_window_slot_add_extra_location_widget (NautilusWindowSlot *slot,
						GtkWidget *widget)
{
	gtk_box_pack_start (GTK_BOX (slot->extra_location_widgets),
			    widget, TRUE, TRUE, 0);
	gtk_widget_show (slot->extra_location_event_box);
}

void
nautilus_window_slot_add_current_location_to_history_list (NautilusWindowSlot *slot)
{

	if ((slot->pane->window == NULL || !NAUTILUS_IS_DESKTOP_WINDOW (slot->pane->window)) &&
	    nautilus_add_bookmark_to_history_list (slot->current_location_bookmark)) {
		nautilus_send_history_list_changed ();
	}
}

/* returns either the pending or the actual current location - used by side panes. */
static char *
real_slot_info_get_current_location (NautilusWindowSlotInfo *info)
{
	NautilusWindowSlot *slot;

	slot = NAUTILUS_WINDOW_SLOT (info);

	if (slot->pending_location != NULL) {
		return g_file_get_uri (slot->pending_location);
	}

	if (slot->location != NULL) {
		return g_file_get_uri (slot->location);
	}

	g_assert_not_reached ();
	return NULL;
}

static NautilusView *
real_slot_info_get_current_view (NautilusWindowSlotInfo *info)
{
	NautilusWindowSlot *slot;

	slot = NAUTILUS_WINDOW_SLOT (info);

	if (slot->content_view != NULL) {
		return g_object_ref (slot->content_view);
	} else if (slot->new_content_view) {
		return g_object_ref (slot->new_content_view);
	}

	return NULL;
}

static void
nautilus_window_slot_dispose (GObject *object)
{
	NautilusWindowSlot *slot;
	GtkWidget *widget;

	slot = NAUTILUS_WINDOW_SLOT (object);

	if (slot->content_view) {
		widget = nautilus_view_get_widget (slot->content_view);
		gtk_widget_destroy (widget);
		g_object_unref (slot->content_view);
		slot->content_view = NULL;
	}

	if (slot->new_content_view) {
		widget = nautilus_view_get_widget (slot->new_content_view);
		gtk_widget_destroy (widget);
		g_object_unref (slot->new_content_view);
		slot->new_content_view = NULL;
	}

	nautilus_window_slot_set_viewed_file (slot, NULL);
	/* TODO? why do we unref here? the file is NULL.
	 * It was already here before the slot move, though */
	nautilus_file_unref (slot->viewed_file);

	if (slot->location) {
		/* TODO? why do we ref here, instead of unreffing?
		 * It was already here before the slot migration, though */
		g_object_ref (slot->location);
	}

	eel_g_list_free_deep (slot->pending_selection);
	slot->pending_selection = NULL;

	if (slot->current_location_bookmark != NULL) {
		g_object_unref (slot->current_location_bookmark);
		slot->current_location_bookmark = NULL;
	}
	if (slot->last_location_bookmark != NULL) {
		g_object_unref (slot->last_location_bookmark);
		slot->last_location_bookmark = NULL;
	}

	if (slot->find_mount_cancellable != NULL) {
		g_cancellable_cancel (slot->find_mount_cancellable);
		slot->find_mount_cancellable = NULL;
	}

	slot->pane = NULL;

	g_free (slot->title);
	slot->title = NULL;

	g_free (slot->status_text);
	slot->status_text = NULL;

	G_OBJECT_CLASS (parent_class)->dispose (object);
}

static void
nautilus_window_slot_info_iface_init (NautilusWindowSlotInfoIface *iface)
{
	iface->active = nautilus_window_slot_active;
	iface->inactive = nautilus_window_slot_inactive;
	iface->get_window = nautilus_window_slot_get_window;
	iface->get_selection_count = nautilus_window_slot_get_selection_count;
	iface->get_current_location = real_slot_info_get_current_location;
	iface->get_current_view = real_slot_info_get_current_view;
	iface->set_status = nautilus_window_slot_set_status;
	iface->get_title = nautilus_window_slot_get_title;
	iface->open_location = nautilus_window_slot_open_location_full;
	iface->make_hosting_pane_active = nautilus_window_slot_make_hosting_pane_active; 
}

